
// MorphCanvas.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, October 2004

/* Create a 3D scene using retained mode (i.e. a scenegraph).
   The scenegraph contains a camera, lights, a blue background,
   a floor (made by Floor), and an morphing morphModel.

   The scene is updated and repainted by a timer task in 
   MorphM3G which calls update() periodically.

   Key presses and releases are passed to the MobileCamera object 
   to affect the camera's position and orientation.
*/

import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class MorphCanvas extends Canvas implements CommandListener 
{
  private Command exitCmd; 

  private Graphics3D iG3D;

  private MorphM3G top;
  private World scene;
  private MobileCamera mobCam;

  // timing information
  private int appTime = 0;
  private int nextTimeToAnimate;


  public MorphCanvas(MorphM3G top) 
  { 
    this.top = top;

    exitCmd = new Command("Exit", Command.EXIT, 0);
    addCommand(exitCmd);
    setCommandListener(this);

    iG3D = Graphics3D.getInstance();
    scene = new World();   // create scene graph
    buildScene();

    // start the animation
    nextTimeToAnimate = scene.animate(appTime);
  }  // end of MorphCanvas()


  public void commandAction(Command command, Displayable disp)
  { 
    if (command == exitCmd)    // finish the morphing
      top.finishGame();
    else   // execution should not get here
      System.out.println("Command Action Error: " + command);
  }


  private void buildScene()
  // add elements to the scene
  {
    addCamera();
    addLights();
    addBackground();

    MorphModel mm = new MorphModel();
    scene.addChild( mm.getModel() );  // add the model

    addFloor();
  } // end of buildScene()


  private void addCamera()
  {
    mobCam = new MobileCamera( getWidth(), getHeight());

    scene.addChild( mobCam.getCameraGroup() );
    scene.setActiveCamera( mobCam.getCamera() ); 
  }  // end of addCamera()


  private void addLights()
  {
    Light light = new Light();  // default white, directional light
    light.setIntensity(1.25f);  // make it a bit brighter
    light.setOrientation(-45.0f, 1.0f, 0, 0); // pointing down and into scene
    scene.addChild(light);
  }  // end of addLights()


  private void addBackground()
  { Background backGnd = new Background();
    backGnd.setColor(0x00bffe); // a light blue background
    scene.setBackground(backGnd);
  }

  private void addFloor()
  {
    Image2D floorIm = loadImage("/bigGrid.gif");   // large, so slow to load
    Floor f = new Floor( floorIm, 8);   // 8 by 8 size

    // Image2D floorIm = loadImage("/grass.gif");    // or try "/floor.png"
    // Floor f = new Floor( floorIm, 6);   // 6 by 6 size

    scene.addChild( f.getFloorMesh() );
  }  // end of addFloor()


  private Image2D loadImage(String fn)
  // load the image stored in fn
  { Image2D im = null;
    try {
       im = (Image2D)Loader.load(fn)[0];
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()


  // ------------ process key presses and releases ------------

  protected void keyPressed(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.pressedKey(gameAction);
  }

  protected void keyReleased(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.releasedKey(gameAction);
  }


  // ------------------ update and paint the canvas ------------------

  public void update()
  // called by a timer task in MorphM3G
  {
    mobCam.update();

    appTime++;
    if (appTime >= nextTimeToAnimate) {
      nextTimeToAnimate = scene.animate(appTime) + appTime;
      // System.out.println("nextTimeToAnimate: " + nextTimeToAnimate);
    }
    repaint();
  }  // end of update()


  protected void paint(Graphics g)
  {
    iG3D.bindTarget(g);
    try {
      iG3D.render(scene);
    }
    catch(Exception e)
    { e.printStackTrace();  }
    finally {
      iG3D.releaseTarget();
    }

    // show the camera's details
    g.drawString( mobCam.getKeyMode(), 5,5, Graphics.TOP|Graphics.LEFT);
    g.drawString( mobCam.getPosition(), 5,18, Graphics.TOP|Graphics.LEFT);
    g.drawString( mobCam.getRotation(), 5,31, Graphics.TOP|Graphics.LEFT);

  } // end of paint()

}  // end of MorphCanvas class
