
// MorphModel.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, October 2004

/* Create an morphing model of a penguin which flaps
   its flippers up and down repeatedly.

   The morphing model is composed of three models whose
   coordinates, normals, and tex coords are stored in
   PenguinBase, PenguinUp, and PenguinDown. PenguinBase
   is the 'base' model, and PenguinUp and PenguinDown 
   are the two 'targets'.

   The only difference between the models is the position
   of the flippers.
*/

import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class MorphModel 
{
  private static final int NUM_TARGETS = 2;
  private static final String MODEL_TEXTURE = "/model.gif";

  private MorphingMesh morphModel;


  public MorphModel() 
  { 
    morphModel = makeMorph();

    // reposition the model's start position and size
    morphModel.setTranslation(0.15f, 0.15f, 0.1f);  // centered at the origin
    morphModel.scale(0.5f, 0.5f, 0.5f);

    setUpAnimation(morphModel);
  }  // end of MorphModel()


  public MorphingMesh getModel()
  {  return morphModel; }


  // ------------------- morphing mesh creation -----------------------------

  private MorphingMesh makeMorph()
  /* The model base is PenguinBase, and its two targets 
     are PenguinUp (flippers up) and PenguinDown (flippers down). 
  */ 
  {
    VertexBuffer base = (new PenguinBase()).getVertexBuffer();

    VertexBuffer[] targets = new VertexBuffer[NUM_TARGETS];  // 2 targets
    targets[0] = (new PenguinUp()).getVertexBuffer();  // PenguinUp
    targets[1] = (new PenguinDown()).getVertexBuffer();  // PenguinDown

    IndexBuffer indBuf = new TriangleStripArray(0, getStripLengths());
    Appearance app = makeAppearance(MODEL_TEXTURE);

    MorphingMesh m = new MorphingMesh(base, targets, indBuf, app);
    return m;
  }  // end of makeMorph()



  private int[] getStripLengths()
  // return an array holding the lengths of each triangle strip;
  // same for the base and targets
  {
    int[] lens = {
      235, 23, 32
    };
    return lens;
  }  // end of getStripLengths()



  private Appearance makeAppearance(String fn)
  // same for the base and targets
  {
    Appearance app = new Appearance();
    app.setMaterial( setMatColours() );

    Image im = loadImage(fn);
    if (im != null) {
      // create an Image2D for the Texture2D
      Image2D image2D = new Image2D(Image2D.RGB, im);

      // create the Texture2D and enable mipmapping
      Texture2D tex = new Texture2D(image2D);

      tex.setFiltering(Texture2D.FILTER_NEAREST, Texture2D.FILTER_NEAREST);
      tex.setWrapping(Texture2D.WRAP_CLAMP, Texture2D.WRAP_CLAMP);
      tex.setBlending(Texture2D.FUNC_MODULATE);  // mix material and texture

      app.setTexture(0, tex);
    }
    return app;
  }  // end of makeAppearance()


  private Material setMatColours()
  // set the material's colour and shininess values
  // Same for the base and targets
  {
    Material mat = new Material();

    mat.setColor(Material.AMBIENT, 0x0003C1C1);
    mat.setColor(Material.EMISSIVE, 0x00000000);
    mat.setColor(Material.DIFFUSE, 0xFFE9B1E8);
    mat.setColor(Material.SPECULAR, 0x00000000);
    mat.setShininess(128.0f);

    return mat;
  } // end of setMatColours()


  private Image loadImage(String fn)
  // load the image stored in fn
  { Image im = null;
    try {
      im = Image.createImage(fn);
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()


  // ---------------- morphing animation ----------------

  private void setUpAnimation(MorphingMesh morphModel)
  /* The animation adjusts the morph weights to switch between the
     base and the targets. */
  {
    // creation animation controller 
    AnimationController animController = new AnimationController();
    animController.setActiveInterval(0, 1500);
    animController.setSpeed(4.0f, 0);	// flap quickly

    // creation morph weights animation track 
    KeyframeSequence morphKS = morphFrames();
    AnimationTrack morphTrack = 
            new AnimationTrack(morphKS, AnimationTrack.MORPH_WEIGHTS); 
    morphTrack.setController(animController);
    morphModel.addAnimationTrack(morphTrack);  // will affect morphModel
  }  // end of setUpAnimation()


  private KeyframeSequence morphFrames()
  {
    KeyframeSequence ks = new KeyframeSequence(8, 2, KeyframeSequence.LINEAR);

    /* Morphing weights make the penguin's flippers flap up, then down,
       then back to the starting base position. First weight is for 
       the up target, the second for the down target.
    */
    ks.setKeyframe(0, 0, new float[]  { 0.0f, 0.0f });  // at base
    ks.setKeyframe(1, 10, new float[] { 0.5f, 0.0f });
    ks.setKeyframe(2, 20, new float[] { 1.0f, 0.0f });  // totally up
    ks.setKeyframe(3, 30, new float[] { 0.5f, 0.0f });
    ks.setKeyframe(4, 40, new float[] { 0.0f, 0.0f });  // back at base
    ks.setKeyframe(5, 50, new float[] { 0.0f, 0.5f });
    ks.setKeyframe(6, 60, new float[] { 0.0f, 1.0f });  // totally down
    ks.setKeyframe(7, 70, new float[] { 0.0f, 0.5f });

    ks.setDuration(80);
    ks.setValidRange(0, 7);
    ks.setRepeatMode(KeyframeSequence.LOOP);

    return ks;
  }  // end of morphFrames()

}  // end of MorphModel class
